@@ -124,7 +124,7 @@ class SVD2DResults(object):
         self.clear_cache()
 
     def __repr__(self):
-        return u'<SVD2DResults: %dx%dx%d>' % (self.u.shape[0], len(self.svals), self.v.shape[0])
+        return u'<SVD2DResults: %d by %d, rank=%d>' % (self.u.shape[0], self.v.shape[0], len(self.svals))
         
     def clear_cache(self):
         '''
@@ -185,11 +185,18 @@ class SVD2DResults(object):
         
         return angles
 
-    def weighted_u_vec(self, idx):
+    def weighted_u_vec(self, idx, default_zeros=False):
         '''
         Get the weighted _u_ vector with key _idx_.
+
+        If _idx_ is not present and default_zeros is true, returns a
+        vector of zeros.
         '''
-        return self.weighted_u[idx, :]
+        try:
+            return self.weighted_u[idx, :]
+        except KeyError:
+            if not default_zeros: raise
+            return DenseTensor(numpy.zeros((len(self.svals),))).labeled([None])
 
     def v_dotproducts_with(self, vec):
         '''
@@ -214,11 +221,18 @@ class SVD2DResults(object):
         
         return angles
 
-    def weighted_v_vec(self, idx):
+    def weighted_v_vec(self, idx, default_zeros=False):
         '''
         Get the weighted _v_ vector with key _idx_.
+
+        If _idx_ is not present and default_zeros is true, returns a
+        vector of zeros.
         '''
-        return self.weighted_v[idx, :]
+        try:
+            return self.weighted_v[idx, :]
+        except KeyError:
+            if not default_zeros: raise
+            return DenseTensor(numpy.zeros((len(self.svals),))).labeled([None])
 
     def get_ahat(self, indices):
         '''
@@ -580,6 +594,6 @@ def incremental_svd(tensor, k=50, niter=100, lrate=.1, logging_interval=None):
     '''
     assert(tensor.ndim == 2)
         
-    u, v, sigma = tensor.isvd(k=k, niter=niter, lrate=lrate)
+    u, v, sigma = tensor._isvd(k=k, niter=niter, lrate=lrate)
     return SVD2DResults(u, v, sigma)
     
