@@ -18,6 +18,7 @@ import copy, random, math
 from csc.divisi.exceptions import DimensionMismatch
 from csc.divisi._svdlib import svd, dictSvd, isvd, dictIsvd
 from warnings import warn
+import logging
 
 __all__ = ['TheEmptySlice',
            'Tensor', 'DictTensor', 'DenseTensor', # Data storage
@@ -25,6 +26,9 @@ __all__ = ['TheEmptySlice',
            'outer_tuple_iterator',  'data', 'wrap_unless_scalar', # Utilities
            ]
 
+def isscalar(obj):
+    return len(getattr(obj, 'shape', ())) == 0
+
 TheEmptySlice = slice(None)
 
 # Subclasses of Tensor need at a bare minimum:
@@ -35,6 +39,7 @@ TheEmptySlice = slice(None)
 # and you can probably make a faster implementation of:
 #  - __len__(): number of keys
 #  - iter_dim_keys()
+#  - inc
 
 class Tensor(object, MyDictMixin):
     """
@@ -66,6 +71,12 @@ class Tensor(object, MyDictMixin):
         with Numpy, this is the same as ``self.ndim``."""
         return self.ndim
 
+    def inc(self, key, amt):
+        '''
+        Increment the value at key by amt.
+        '''
+        self[key] += amt
+        
     def __len__(self):
         """
         The number of *entries* stored in this tensor.
@@ -206,7 +217,7 @@ class Tensor(object, MyDictMixin):
         return res
         
     def __iadd__(self, other):
-        if numpy.isscalar(other):
+        if isscalar(other):
             return self.scalar_iadd(other)
         else:
             return self.tensor_iadd(other)
@@ -225,7 +236,7 @@ class Tensor(object, MyDictMixin):
         If *other* is a scalar, this will perform scalar multiplication
         instead.
         """
-        if numpy.isscalar(other):
+        if isscalar(other):
             return self.cmul(other)
         else:
             return self.dot(other)
@@ -284,6 +295,10 @@ class Tensor(object, MyDictMixin):
 
     magnitude = norm
 
+
+    ## Vector operations
+    # Most of the operations below only make sense on vectors.
+    
     def normalized_copy(self):
         '''
         Return a normalized copy of this tensor. For a vector, this is
@@ -299,6 +314,7 @@ class Tensor(object, MyDictMixin):
         assert self.ndim == 1
         return (self * other) / (other * other) * other
 
+    
     def components_wrt(self, other):
         '''
         Return the (parallel, perpendicular) components of this vector
@@ -307,6 +323,18 @@ class Tensor(object, MyDictMixin):
         projection = self.projection_onto(other)
         return projection, (self-projection)
 
+    def cosine_of_angle_to(self, other):
+        '''
+        Return the cosine of the angle of this vector with another.
+
+        A dot B = |A| |B| cos \theta
+        => cos \theta = (A dot B) / (|A| |B|)
+        '''
+        if self.ndim != 1: raise TypeError('Must be a vector')
+        return (self*other) / (self.norm() * other.norm())
+    
+
+    ## Summarization operations
     def extremes(self, n=10, top_only=False):
         """
         Extract the interesting parts of this tensor.
@@ -321,12 +349,24 @@ class Tensor(object, MyDictMixin):
         If this is a vector (as it often is), note that the indices are going
         to be tuples with one thing in them, for consistency.
         """
-        import operator
-        pairs = sorted(self.iteritems(), key=operator.itemgetter(1))
+        from heapq import heapreplace
+        from sys import maxint
+        top = [(-maxint,None)] * n
+        bottom = [(-maxint,None)] * n # maintains the negative of the bottom part
+
+        for k, v in self.iteritems():
+            if v > top[0][0]: heapreplace(top, (v, k))
+            if -v > bottom[0][0]: heapreplace(bottom, (-v, k))
+
+        top.sort()
+        bottom.sort(reverse=True)
+        top = [(k, v) for v, k in top]
+        bottom = [(k, -v) for v, k in bottom]
+
         if top_only:
-            return pairs[:n]
+            return top
         else:
-            return pairs[:n] + pairs[-n:]
+            return bottom + top
 
     def show_extremes(self, n=10, output=sys.stdout):
         """
@@ -357,7 +397,7 @@ class Tensor(object, MyDictMixin):
     def _svd(self, k=50, *a, **kw):
         return svd(self, k=k, *a, **kw)
 
-    def isvd(self, k=50, niter=100, lrate=.001, *a, **kw):
+    def _isvd(self, k=50, niter=100, lrate=.001, *a, **kw):
         return isvd(self, k=k, niter=niter, lrate=lrate, *a, **kw)
 
 class InvalidLayeringException(StandardError):
@@ -391,13 +431,16 @@ class View(Tensor):
 
     def __getattr__(self, name):
         '''Fall back to the tensor operation.'''
-        if name.startswith('__') or name in attribute_blacklist:
+        if name.startswith('_') or name in attribute_blacklist:
             raise AttributeError(name)
         return getattr(self.tensor, name)
 
     def __iter__(self):
         return self.tensor.__iter__()
 
+    def __len__(self):
+        return len(self.tensor)
+
     def add_layer(self, cls, *a, **kw):
         """
         Given a class of View, determine where it should go in the stack
@@ -531,6 +574,18 @@ class DictTensor(Tensor):
                         yield x
         return iter_helper(self._data, self.ndim, ())
 
+    def iteritems(self):
+        def iter_helper(dict, num_nested, key_base):
+            if num_nested == 1:
+                for x, v in dict.iteritems():
+                    # TODO: is using append here too slow?
+                    yield (key_base + (x,)), v
+            else:
+                for key, child_dict in dict.iteritems():
+                    for x in iter_helper(child_dict, num_nested - 1, key_base + (key,)):
+                        yield x
+        return iter_helper(self._data, self.ndim, ())
+    
     @property
     def shape(self):
         """
@@ -804,7 +859,7 @@ class DictTensor(Tensor):
     def _svd(self, k=50, *a, **kw):
         return dictSvd(self, k=k, *a, **kw)
 
-    def isvd(self, k=50, niter=100, lrate=.001, *a, **kw):
+    def _isvd(self, k=50, niter=100, lrate=.001, *a, **kw):
         return dictIsvd(self, k=k, niter=niter, lrate=lrate, *a, **kw)
 
 try:
@@ -845,7 +900,7 @@ def data(tensor):
     return getattr(tensor, '_data', tensor)
 
 def wrap_unless_scalar(result):
-    if numpy.isscalar(result): return result
+    if isscalar(result): return result
     else: return DenseTensor(result)
 
 class DenseTensor(Tensor):
@@ -972,7 +1027,7 @@ class DenseTensor(Tensor):
         newdata = numpy.concatenate((self._data, other._data))
         return DenseTensor(newdata)
 
-    def k_means_inner(self, k=20):
+    def k_means_inner(self, k, tolerance, iterlim):
         m, n = self.shape
         
         # Normalize the data so that we're working with directions
@@ -1001,7 +1056,8 @@ class DenseTensor(Tensor):
         error = 1000.0
         dist = 1000.0
         clusters = numpy.zeros((m,), 'i')
-        while error > numpy.linalg.norm(means)/1000:
+        iteration = 0
+        while error > numpy.linalg.norm(means)*tolerance:
             # Find the closest mean for each row
             dots = numpy.dot(directions, means.T)
             dist = 0.0
@@ -1026,6 +1082,11 @@ class DenseTensor(Tensor):
 
             error = numpy.linalg.norm(newmeans - means)
             means = newmeans
+
+            iteration += 1
+            if iteration > iterlim:
+                logging.warn('k_means iteration limit exceeded')
+                break
         
         clusterlists = [[] for i in xrange(k)]
         for row in xrange(m):
@@ -1034,7 +1095,7 @@ class DenseTensor(Tensor):
 
         return DenseTensor(means), clusterlists, dist
 
-    def k_means(self, k=20, iter=10):
+    def k_means(self, k=20, iter=10, tolerance=0.001, iterlim=1000):
         """
         Divide the data into clusters using k-means clustering.
 
@@ -1045,7 +1106,7 @@ class DenseTensor(Tensor):
         best_results = (None, None)
         best_dist = 1e300
         for iteration in xrange(iter):
-            means, clusters, dist = self.k_means_inner(k)
+            means, clusters, dist = self.k_means_inner(k, tolerance, iterlim)
             print dist
             if dist < best_dist:
                 best_dist = dist
