@@ -10,162 +10,10 @@ from math import sqrt
 
 import numpy
 import random
-import sys
 
-def dict_fill_in_missing_dims(d):
-    items = d.items()
-    ndim = len(items[0])
-    dim_keys = [set() for i in xrange(ndim)]
+from tensor_util import assert_dims_consistent, assertTensorEqual, assertTensorEqualDict, nones_removed
 
-    for cur_dim in xrange(ndim):
-        for key, value in items:
-            dim_keys[cur_dim].add(key[cur_dim])
-
-    def enumerate_keys(dim_keys):
-        if len(dim_keys) == 0:
-            yield ()
-        else:
-            for key_end in enumerate_keys(dim_keys[1:]):
-                for key_part in dim_keys[0]:
-                    yield (key_part,) + key_end
-
-    new_dict = {}
-    for key in enumerate_keys(dim_keys):
-        new_dict[key] = d.get(key, None)
-
-    return new_dict
-
-def nones_removed(d):
-    return dict((k, v) for k, v in d.iteritems() if v is not None)
-
-
-def get_shape_for_dict(expected):
-    if len(expected.keys()) == 0:
-        return ()
- 
-    ndim = len(expected.keys()[0])
-    return tuple(len(set(k[dim] for k in expected.iterkeys()))
-                 for dim in xrange(ndim))
-
-
-class TensorTest(unittest.TestCase):
-    def assertDimsConsistent(self):
-        self.assertEqual(self.tensor.ndim, len(self.tensor.shape))
-    
-    def assertShapeEqual(self, actual, expected):
-        expected_shape = get_shape_for_dict(expected)
-        self.assertEqual(expected_shape, actual.shape)
-        self.assertEqual(len(expected_shape), actual.dims)
-        self.assertEqual(len(expected_shape), actual.ndim)
-
-    def assertValuesEqual(self, actual, expected):
-        '''
-        Test which values are specified in the tensor.
-
-        Args:
-         actual: the Tensor result
-         expected: a dictionary of expected results
-        '''
-        for index, expected_value in expected.iteritems():
-            if expected_value is not None:
-                self.assertTrue(index in actual)
-                self.assertTrue(actual.has_key(index))
-                self.assertAlmostEqual(actual[index], expected_value)
-            else:
-                self.assertFalse(index in actual)
-                self.assertFalse(actual.has_key(index))        
-                self.assertAlmostEqual(actual[index], actual.default_value)
-
-        # Test some the shape boundary condition
-        self.assertFalse(get_shape_for_dict(expected) in actual)
-
-    def assertItemsCorrect(self, actual, expected):
-        # Filter just what's specified.
-        specified = nones_removed(expected)
-
-        # Test __iter__
-        specified_keys = specified.keys()
-        specified_keys.sort()
-        indices = list(iter(actual))
-
-        self.assertEqual(sorted(indices), specified_keys)
-        self.assertEqual(sorted(actual.keys()), specified_keys)
-
-        # Same test for iteritems()
-        items_iter = sorted(list(actual.iteritems()))
-        items_noniter = sorted(actual.items())
-        specified_items = sorted(specified.items())
-        # Values are subject to floating-point error, so an
-        # AlmostEqual test is required.
-        for (actual_key, actual_val), (expected_key, expected_val) \
-                in zip(items_iter, specified_items):
-            self.assertEqual(actual_key, expected_key)
-            self.assertAlmostEqual(actual_val, expected_val)
-        for (actual_key, actual_val), (expected_key, expected_val) \
-                in zip(items_noniter, specified_items):
-            self.assertEqual(actual_key, expected_key)
-            self.assertAlmostEqual(actual_val, expected_val)
-
-        # Test the values() method
-        specified_values = specified.values()
-        for a, e in zip(sorted(actual.values()), sorted(specified_values)):
-            self.assertAlmostEqual(a, e)
-
-        # Test that the length of the tensor is equal to the number of specified items
-        self.assertEqual(len(actual), len(specified_keys))
-
-        # Test the __contains__ method
-        self.assertTrue(specified_keys[0] in actual)
-        # can't easily test the contrapositive, since we can't easily make up an index that's not in there.
-
-    def assertIterDimKeysCorrect(self, actual, expected):
-        specified_indices = [k for k, v in expected.iteritems()
-                             if v is not None]
-        for dim in xrange(len(specified_indices[0])):
-            expected_keys = set(key[dim] for key in specified_indices)
-            dim_keys = list(actual.iter_dim_keys(dim))
-            self.assertEqual(sorted(dim_keys), sorted(expected_keys))
-
-    def assertTensorEqual(self, actual, expected):
-        '''Takes an n-dimensional array 'expected' and runs several
-        tests on the tensor 'actual'. For the test description, see
-        assertTensorEqualCompleteDict.
-        '''
-        expected_dict = nested_list_to_dict(expected)
-        self._assertTensorEqualCompleteDict(actual, expected_dict)
-
-    def assertTensorEqualDict(self, actual, expected_dict):
-        expected_complete_dict = dict_fill_in_missing_dims(expected_dict)
-        self._assertTensorEqualCompleteDict(actual, expected_complete_dict)
-
-    def _assertTensorEqualCompleteDict(self, actual, expected_dict):
-        '''
-        Takes a tensor and a dictionary of key => value pairs, e.g.
-        {(0,0) : 1, (0, 1) : 2}
-        and performs the following tests:
-
-        1. actual's dimension and shape match that of expected
-        2. Every index of actual has the same value as expected.
-        3. Every index of expected is contained (i.e. using python's "in")
-           in actual (Note: values in expected can be None, in which case
-           actual is assumed to not contain the index and 
-           the value of actual at that index is the tensor's default
-           value)
-        4. Iteration (__iter__ and iteritems) correctly iterate over 
-           only the specified indices of actual
-        5. iter_dim_keys iterates over specified keys of the specified dimension
-        6. keys() and values() methods return only the specified indices
-           of actual
-        7. len(actual) returns the number of specified indices in expected
-        8. has_key returns True for specified indices and False for unspecified indices
-        '''
-        self.assertShapeEqual(actual, expected_dict)
-        self.assertValuesEqual(actual, expected_dict)
-        self.assertItemsCorrect(actual, expected_dict)
-        self.assertIterDimKeysCorrect(actual, expected_dict)
-
-
-class DictTensorTest(TensorTest):
+class DictTensorTest(unittest.TestCase):
     slice_testcase = [[1,    None, None],
                        [None, 2,    3   ],
                        [4,    None, None],
@@ -174,9 +22,9 @@ class DictTensorTest(TensorTest):
     def test_initial(self):
         self.assertEqual(len(self.tensor), 0)
         self.assertEqual(len(self.tensor.keys()), 0)
-        self.assertDimsConsistent()
+        assert_dims_consistent(self.tensor)
         self.assertEqual(self.tensor.shape, (0, 0))
-        self.assert_(isinstance(self.tensor[4, 5], (float, int, long)))
+        assert isinstance(self.tensor[4, 5], (float, int, long))
         self.assertEqual(self.tensor[5, 5], 0)
         self.assertEqual(self.tensor[2, 7], 0)
 
@@ -184,13 +32,13 @@ class DictTensorTest(TensorTest):
         self.tensor[5, 5] = 1
         self.tensor[2, 7] = 2
 
-        self.assertTensorEqual(self.tensor,
-                               [[None]*8,
-                                [None]*8,
-                                [None]*7 + [2],
-                                [None]*8,
-                                [None]*8,
-                                [None]*5 + [1, None, None]])
+        assertTensorEqual(self.tensor,
+                          [[None]*8,
+                           [None]*8,
+                           [None]*7 + [2],
+                           [None]*8,
+                           [None]*8,
+                           [None]*5 + [1, None, None]])
 
     def test_slice(self):
         self.tensor.update(nones_removed(nested_list_to_dict(self.slice_testcase)))
@@ -198,18 +46,18 @@ class DictTensorTest(TensorTest):
         # Test end conditions: start index
         # is included in slice, end index is not
         slice = self.tensor[1:3, 0:2]
-        self.assertTensorEqual(slice,
-                               [[None, 2],
-                                [4, None]])
+        assertTensorEqual(slice,
+                          [[None, 2],
+                           [4, None]])
 
         # Test that slicing on some dims correctly 
         # reduces the dimensionality of the tensor
         slice = self.tensor[3, :]
-        self.assertTensorEqual(slice, [None, 5, None])
+        assertTensorEqual(slice, [None, 5, None])
 
         # Test the step parameter
         slice = self.tensor[1:4:2, :]
-        self.assertTensorEqual(slice, 
+        assertTensorEqual(slice,
                                [[None, 2, 3],
                                 [None, 5, None]])
 
@@ -220,17 +68,17 @@ class DictTensorTest(TensorTest):
         self.tensor[3, 1] = 5
 
         t = self.tensor.transpose()
-        self.assertTensorEqual(t,
-                               [[1, None, 4, None],
-                                [None, None, None, 5],
-                                [None, 3, None, None]])
-
+        assertTensorEqual(t,
+                          [[1, None, 4, None],
+                           [None, None, None, 5],
+                           [None, 3, None, None]])
+        
     def test_delete(self):
         self.tensor.update(nones_removed(nested_list_to_dict(self.slice_testcase)))
-        self.assertTensorEqual(self.tensor, self.slice_testcase)
+        assertTensorEqual(self.tensor, self.slice_testcase)
 
         del self.tensor[0,0]
-        self.assertTensorEqual(self.tensor,
+        assertTensorEqual(self.tensor,
                                [[None, None, None],
                                 [None, 2,    3   ],
                                 [4,    None, None],
@@ -252,7 +100,7 @@ class DictTensorTest(TensorTest):
         tensor_1D = DictTensor(1)
         tensor_1D[2] = 1 
 
-        self.assertTensorEqual(tensor_1D,
+        assertTensorEqual(tensor_1D,
                                [None, None, 1])
 
     def test_combine_by_element(self):
@@ -264,7 +112,7 @@ class DictTensorTest(TensorTest):
         t2[0, 1] = 5
 
         t3 = t1.combine_by_element(t2, lambda x, y: x + (2*y))
-        self.assertTensorEqual(t3,
+        assertTensorEqual(t3,
                                [[None, 10],
                                 [2, 9]])
 
@@ -287,7 +135,7 @@ class DictTensorTest(TensorTest):
         t2[1, 0] = 5
 
         t3 = t1 + t2
-        self.assertTensorEqual(t3,
+        assertTensorEqual(t3,
                                [[1, None],
                                 [7, 1],
                                 [None, 4]])
@@ -306,14 +154,14 @@ class DictTensorTest(TensorTest):
 
             
 from numpy import zeros
-class DenseTensorTest(TensorTest):
+class DenseTensorTest(unittest.TestCase):
     def setUp(self):
         self.tensor = DenseTensor(zeros((3,4)))
 
     def test_initial(self):
         self.assertEqual(len(self.tensor), 3*4)
         self.assertEqual(len(self.tensor.keys()), 3*4)
-        self.assertDimsConsistent()
+        assert_dims_consistent(self.tensor)
         self.assert_(isinstance(self.tensor[2, 3], (float, int, long)))
         self.assertEqual(self.tensor.shape, (3, 4))
         self.assertEqual(self.tensor[1, 2], 0)
@@ -324,110 +172,13 @@ class DenseTensorTest(TensorTest):
         self.tensor[2, 3] = 2
         self.tensor[0, -1] = 3
 
-        self.assertTensorEqual(self.tensor,
+        assertTensorEqual(self.tensor,
                                [[1, 0, 0, 3],
                                 [0, 0, 0, 0],
                                 [0, 0, 0, 2]])
 
-class DenseVectorTest(TensorTest):
-    def setUp(self):
-        self.tensor = DenseTensor(zeros((5,)))
 
-    def testDotProduct(self):
-        # Set up the other tensor
-        tensor2 = DenseTensor(zeros(5,))
-        # Dot them
-        result = self.tensor*tensor2
-        self.assertEqual(result, 0)
-
-    def testAdding(self):
-        # Set up the other tensor
-        tensor2 = DenseTensor(zeros(5,))
-        tensor2[3] = 3
-        # Add them.
-        result = self.tensor + tensor2
-        self.assert_((result._data == [0,0,0,3,0]).all())
-
-    def testAddDims(self):
-        self.assertRaises(ValueError, lambda: self.tensor + DenseTensor(zeros(3,)))
-        self.assertRaises(TypeError, lambda: self.tensor + DenseTensor(zeros(5,5)))    
-
-    def testVectorNorm(self):
-        tensor = self.tensor
-        tensor[0] = 1
-        tensor[1] = 5
-        self.assertAlmostEqual(tensor.norm(), sqrt(26.0))
-        normalized = self.tensor.normalized_copy()
-        self.assertAlmostEqual(normalized*normalized, 1.0)
-        
-
-
-class LabeledDenseTensorTest(TensorTest):
-    labels = [['a', 'b', 'c']]
-
-    def setUp(self):
-        self.raw = DenseTensor(zeros((3,)))
-        self.tensor = LabeledView(self.raw, self.labels)
-        self.tensor['c'] = 3
-
-    def testAdding(self):
-        # Set up the other tensor
-        raw2 = DenseTensor(zeros((3,)))
-        tensor2 = LabeledView(raw2, self.labels)
-        tensor2['a'] = 2
-        
-        # Add.
-        result = self.tensor + tensor2
-        self.assertEqual(result['a'], 2)
-        self.assertEqual(result['b'], 0)
-        self.assertEqual(result['c'], 3)
-
-    def testAddMismatch(self):
-        # Set up the other tensor
-        raw2 = DenseTensor(zeros((3,)))
-        tensor2 = LabeledView(raw2, [['q','r','s']])
-        tensor2['q'] = 2
-
-        # Should fail because the new dense tensor would have to be bigger
-        self.assertRaises(IndexError, lambda: self.tensor + tensor2)
-
-    def test_iter_dim_keys(self):
-        raw = DenseTensor(zeros((2, 3)))
-        labels = [['a', 'b'], ['c', 'd', 'e']]
-        tensor = LabeledView(raw, labels)
-
-        i = 0
-        for key in tensor.iter_dim_keys(0):
-            self.assertEqual(key, labels[0][i])
-            i += 1
-        self.assertEqual(i, 2)
-        
-        i = 0
-        for key in tensor.iter_dim_keys(1):
-            self.assertEqual(key, labels[1][i])
-            i += 1
-        self.assertEqual(i, 3)
-
-    def test_combine_by_element(self):
-        t1 = LabeledView(DenseTensor(zeros((2,2))), [['a', 'b'], ['c', 'd']])
-        t2 = LabeledView(DenseTensor(zeros((2,2))), [['a', 'b'], ['c', 'd']])
-        t1['a', 'c'] = 1
-        t1['b', 'c'] = 2
-        t2['a', 'c'] = 4
-        t2['a', 'd'] = 5
-
-        t3 = t1.combine_by_element(t2, lambda x, y: x + (2*y))
-        self.assertEqual(t3['a', 'c'], 9)
-        self.assertEqual(t3['b', 'c'], 2)
-        self.assertEqual(t3['a', 'd'], 10)
-        self.assertEqual(t3['b', 'd'], 0)
-
-        t4 = DenseTensor(zeros((3, 2)))
-        self.assertRaises(IndexError, lambda: t1.combine_by_element(t4, lambda x, y: x + y))
-        t4 = DenseTensor(zeros((2, 2, 1)))
-        self.assertRaises(IndexError, lambda: t1.combine_by_element(t4, lambda x, y: x + y))
-
-class LabeledDictTensorTest(TensorTest):
+class LabeledDictTensorTest(unittest.TestCase):
     def setUp(self):
         self.tensor = make_sparse_labeled_tensor(2)
 
@@ -450,7 +201,7 @@ class LabeledDictTensorTest(TensorTest):
                     ('apple', 'red') : 2,
                     ('apple', 'blue') : 3,
                     ('orange', 'yellow') : 4}
-        self.assertTensorEqualDict(self.tensor, expected)
+        assertTensorEqualDict(self.tensor, expected)
 
 
     def test_slicing(self):
@@ -513,14 +264,14 @@ normalize_testcase = [[1, None],
 normalize_expected_result = [[1, None],
                              [3/5., 4/5.]]
 
-class NormalizedViewTest(TensorTest):
+class NormalizedViewTest(unittest.TestCase):
     def setUp(self):
         self.raw = DictTensor(2)
         self.raw.update(nones_removed(nested_list_to_dict(normalize_testcase)))
         self.tensor = NormalizedView(self.raw, 0)
 
     def test_result(self):
-        self.assertTensorEqual(self.tensor, normalize_expected_result)
+        assertTensorEqual(self.tensor, normalize_expected_result)
 
     def test_contains(self):
         self.assertTrue((0,0) in self.tensor)
@@ -528,7 +279,7 @@ class NormalizedViewTest(TensorTest):
         self.assertFalse((0,1) in self.tensor)
         self.assertFalse(self.tensor.has_key((0,1)))
 
-class ZeroMeanViewTest(TensorTest):
+class ZeroMeanViewTest(unittest.TestCase):
     def setUp(self):
         self.tensor = DictTensor(ndim=2)
         self.tensor[0, 3] = 2
@@ -568,7 +319,7 @@ class ZeroMeanViewTest(TensorTest):
 
 
 
-class TfIdfTest(TensorTest):
+class TfIdfTest(unittest.TestCase):
     def test(self):
         '''Run the testcase from the Wikipedia article (in comments)'''
         tensor = DictTensor(2)
@@ -754,7 +505,7 @@ class TestMult(unittest.TestCase):
             for key, val in result.iteritems():
                 self.assertAlmostEqual(val, result_s[key])
 
-class NormalizeTest(TensorTest):
+class NormalizeTest(unittest.TestCase):
     data = [[-5, 0, 0, 0],
             [0, 1, -2, 3],
             [100, 2, 0, 0]]
@@ -783,7 +534,7 @@ svd_2d_test_matrix[0, 4] = 2
 svd_2d_test_matrix[1, 2] = 3
 svd_2d_test_matrix[3, 1] = 4
 
-class SVD2DTest(TensorTest):
+class SVD2DTest(unittest.TestCase):
     def setUp(self):
         self.tensor = DictTensor(2)
         # Note: this command actually puts 20 values in tensor!
@@ -798,20 +549,20 @@ class SVD2DTest(TensorTest):
         self.assertEqual(len(self.incremental.svals), self.incremental.v.shape[1])
         self.assertEqual(self.incremental.v.shape[0], self.tensor.shape[1])
 
-        self.assertTensorEqual(self.incremental.u, 
+        assertTensorEqual(self.incremental.u, 
                                [[0, 0, 1],
                                 [0, 1, 0],
                                 [0, 0, 0],
                                 [1, 0, 0]])
 
-        self.assertTensorEqual(self.incremental.v, 
+        assertTensorEqual(self.incremental.v, 
                                [[0, 0, sqrt(.2)],
                                 [1, 0, 0],
                                 [0, 1, 0],
                                 [0, 0, 0],
                                 [0, 0, sqrt(.8)]])
 
-        self.assertTensorEqual(self.incremental.svals,
+        assertTensorEqual(self.incremental.svals,
                                [4, 3, sqrt(5)])
 
     def test_decomposition(self):
@@ -820,41 +571,41 @@ class SVD2DTest(TensorTest):
         self.assertEqual(len(self.svals), self.v.shape[1])
         self.assertEqual(self.v.shape[0], self.tensor.shape[1])
 
-        self.assertTensorEqual(self.u, 
+        assertTensorEqual(self.u, 
                                [[0, 0, 1],
                                 [0, -1, 0],
                                 [0, 0, 0],
                                 [-1, 0, 0]])
 
-        self.assertTensorEqual(self.v, 
+        assertTensorEqual(self.v, 
                                [[0, 0, sqrt(.2)],
                                 [-1, 0, 0],
                                 [0, -1, 0],
                                 [0, 0, 0],
                                 [0, 0, sqrt(.8)]])
 
-        self.assertTensorEqual(self.svals,
+        assertTensorEqual(self.svals,
                                [4, 3, sqrt(5)])
 
     def test_reconstructed(self):
-        self.assertTensorEqual(self.svd.reconstructed,
+        assertTensorEqual(self.svd.reconstructed,
                                [[1, 0, 0, 0, 2],
                                 [0, 0, 3, 0, 0],
                                 [0, 0, 0, 0, 0],
                                 [0, 4, 0, 0, 0]])
-        self.assertTensorEqual(self.svd.reconstructed[1,:],
+        assertTensorEqual(self.svd.reconstructed[1,:],
                                 [0, 0, 3, 0, 0])
-        self.assertTensorEqual(self.svd.reconstructed[:,2],
+        assertTensorEqual(self.svd.reconstructed[:,2],
                                [0, 3, 0, 0])
 
     def test_orthonormality(self):
         identity = [[1, 0, 0],
                      [0, 1, 0],
                      [0, 0, 1]]
-        self.assertTensorEqual(self.u.T * self.u,
+        assertTensorEqual(self.u.T * self.u,
                                identity)
 
-        self.assertTensorEqual(self.v.T * self.v,
+        assertTensorEqual(self.v.T * self.v,
                                identity)
     
     def test_variance(self):        
@@ -870,7 +621,7 @@ class SVD2DTest(TensorTest):
         diff_k1_mag = diff_k1.magnitude()
         self.assert_(tensor_mag > diff_k1_mag > diff_k3_mag)
 
-class NormalizedSVD2DTest(TensorTest):
+class NormalizedSVD2DTest(unittest.TestCase):
     def setUp(self):
         self.tensor = DictTensor(2)
         self.tensor.update(nested_list_to_dict(
@@ -893,8 +644,8 @@ class NormalizedSVD2DTest(TensorTest):
         pass # TODO
 
     def test_orthonormality(self):
-        self.assertTensorEqual(self.u.T * self.u, numpy.eye(self.u.shape[1]))
-        self.assertTensorEqual(self.v.T * self.v, numpy.eye(self.u.shape[1]))
+        assertTensorEqual(self.u.T * self.u, numpy.eye(self.u.shape[1]))
+        assertTensorEqual(self.v.T * self.v, numpy.eye(self.u.shape[1]))
     
     def test_variance(self):        
         return # TODO
@@ -919,7 +670,7 @@ normalize_testcase3 = [[70, 0],
 
 
 
-class NormalizedSVD2DTest2(TensorTest):
+class NormalizedSVD2DTest2(unittest.TestCase):
     def assertItemsEqual(self, t1, t2):
         for key in t1.iterkeys():
             self.assertAlmostEqual(t1[key], t2[key])
